/*
 *  Copyright (c) Members of the EGEE Collaboration. 2005.
 *  See http://eu-egee.org/partners/ for details on the copyright holders.
 *  For license conditions see the license file or http://eu-egee.org/license.html
 *
 *  GLite Data Management - Simple file transfer stats API
 *
 *  Authors: Zoltan Farkas <Zoltan.Farkas@cern.ch>
 *
 */

#ifndef GLITE_DATA_TRANSFER_TRANSFERSTATS_SIMPLE_H
#define GLITE_DATA_TRANSFER_TRANSFERSTATS_SIMPLE_H

#ifdef __cplusplus
extern "C" {
#endif

#include <glite/data/transfer/c/transfer-simple.h>

/**********************************************************************
 * Constants
 */

/* Service type for Service Discovery */
#define GLITE_TRANSFERSTATS_SD_TYPE	"org.glite.FileTransferStats"

/* Environment variable to override default service type */
#define GLITE_TRANSFERSTATS_SD_ENV	"GLITE_SD_STATS_TYPE"


/**********************************************************************
 * General guidelines:
 * - Functions that return a pointer return NULL when there is an error.
 * - Functions that return 'int' return 0 when successful and -1 in case
 *   of an error.
 * - Any objects returned by a function is owned by the caller and has to
 *   be deallocated by the caller.
 */

/**********************************************************************
 * Function prototypes - library management functions
 */

/* Return the current FileTransferStats endpoint used by the transfer context */
const char *glite_transferstats_get_endpoint(glite_transfer_ctx *ctx);

/**********************************************************************
 * Function prototypes - org.glite.data.transfer.ServiceBase
 */

/**
 * @defgroup transfer_ServiceBase org.glite.data.transfer.ServiceBase
 * @{
 */

/**
 * Get the version of the remote endpoint.
 * 
 * @param ctx The transfer context.
 *
 * @return The version string, or NULL in case of an error. 
 */
char *glite_transferstats_getVersion(glite_transfer_ctx *ctx);

/**
 * Get the schema version of the remote endpoint.
 *
 * @param ctx The transfer context.
 *
 * @return The schema version string, or NULL in case of an error. 
 */
char *glite_transferstats_getSchemaVersion(glite_transfer_ctx *ctx);

/**
 * Get the interface version of the remote endpoint.
 * 
 * @param ctx The transfer context.
 *
 * @return The interface version string, or NULL in case of an error. 
 */
char *glite_transferstats_getInterfaceVersion(glite_transfer_ctx *ctx);

/**
 * Get the service metadata for a given key.
 * 
 * @param ctx The transfer context.
 * @param key The key of which the metadata we want to get. Must not be NULL.
 *
 * @return The metadata string, or NULL in case of an error. 
 */
char *glite_transferstats_getServiceMetadata(glite_transfer_ctx *ctx, const char *key);

/**
 * @}
 */

/***********************************************************************
 * Function prototypes - org.glite.data.transfer.stats.FileTransferStats
 */

/**
 * @defgroup FileTransferStats org.glite.data.transfer.stats.FileTransferStats
 * @{
 */

/**
 * Get Channel usage summary of a service on a given interval of time
 * 
 * @param ctx The transfer context.
 * @param startTime The start time.
 * @param endTime The end time.
 * @param channelName The name of the channel.
 * @param numVONames Size of array VONames.
 * @param VONames A list of VOs of interest for which summaries should be retrieved. Null means all VOs.
 * @param Summary [OUT] Array of TransferSummary objects.
 *
 * @return Size of Summary array, and -1 in case of errors.
 */
int glite_transferstats_getChannelSummary(glite_transfer_ctx *ctx,
	const time_t startTime, const time_t endTime, const char *channelName,
	const int numVONames, const char **VONames, glite_transfer_TransferSummary ***Summary);

/**
 * Get Channel activity on a given interval of time
 * 
 * @param ctx The transfer context.
 * @param startTime The start time.
 * @param endTime The end time.
 * @param channelName The name of the channel.
 * @param numVONames Size of array VONames.
 * @param VONames A list of VOs of interest for which activities should be retrieved. Null means all VOs.
 * @param Activity [OUT] Array of TransferActivity objects.
 *
 * @return Size of Activity array, and -1 in case of errors.
 */
int glite_transferstats_getChannelActivity(glite_transfer_ctx *ctx,
	const time_t startTime, const time_t endTime, const char *channelName,
	const int numVONames, const char **VONames, glite_transfer_TransferActivity ***Activity);

/**
 * Get VO usage summary of a service on a given interval of time
 * 
 * @param ctx The transfer context.
 * @param startTime The start time.
 * @param endTime The end time.
 * @param voName The name of the channel.
 * @param numchannelNames Size of array channelNames.
 * @param channelNames A list of channels of interest for which summaries should be retrieved. Null means all channels.
 * @param Summary [OUT] Array of TransferSummary objects.
 *
 * @return Size of Summary array, and -1 in case of errors.
 */
int glite_transferstats_getVOSummary(glite_transfer_ctx *ctx,
	const time_t startTime, const time_t endTime, const char *voName,
	const int numchannelNames, const char **channelNames, glite_transfer_TransferSummary ***Summary);

/**
 * Get VO activity on a given interval of time
 * 
 * @param ctx The transfer context.
 * @param startTime The start time.
 * @param endTime The end time.
 * @param voName The name of the VO.
 * @param numchannelNames Size of array channelNames.
 * @param channelNames A list of channels of interest for which activities should be retrieved. Null means all channels.
 * @param Activity [OUT] Array of TransferActivity objects.
 *
 * @return Size of Activity array, and -1 in case of errors.
 */
int glite_transferstats_getVOActivity(glite_transfer_ctx *ctx,
	const time_t startTime, const time_t endTime, const char *voName,
	const int numchannelNames, const char **channelNames, glite_transfer_TransferActivity ***Activity);

/**
 * Get list of agents running on the service
 * 
 * @param ctx The transfer context.
 * @param Agents [OUT] The return value
 *
 * @return Size of the Agents array, -1 in case of errors.
 */
int glite_transferstats_getTransferAgents(glite_transfer_ctx *ctx,
	glite_transfer_TransferAgent ***Agents);

/**
 * @}
 */

#ifdef __cplusplus
}
#endif

#endif /* GLITE_DATA_TRANSFER_TRANSFERSTATS_SIMPLE_H */
